// Simple blinking led program for the LPC-P2148 board

/* *********************************************************
               Function declarations
  ********************************************************* */

void Initialize(void);
void feed(void);

void IRQ_Routine (void)   __attribute__ ((interrupt("IRQ")));
void FIQ_Routine (void)   __attribute__ ((interrupt("FIQ")));
void SWI_Routine (void)   __attribute__ ((interrupt("SWI")));
void UNDEF_Routine (void) __attribute__ ((interrupt("UNDEF")));

/**********************************************************
                  Header files
 **********************************************************/
#include "LPC2378.h"

/**********************************************************
                       MAIN
**********************************************************/

void InitMAM(void) {

  MAMCR = 0;				// disable MAM
  MAMTIM = 4;    // FCLK > 40 MHz
  MAMCR = 1;   // MAM functions fully enabled

}

/*************************************************************************
 * Function Name: InitClock
 * Parameters: void
 * Return: void
 *
 * Description: Initialize PLL and clocks' dividers. Pclk - 144MHz,
 * Cclk- 72MHz, Usbclk - 48MHz
 *
 *************************************************************************/
void InitClock(void)
{
  // 1. Init OSC
  //SCS_bit.OSCRANGE = 0;
  //SCS_bit.OSCEN = 1;
	SCS &= ~(1<<4); // OSCRANGE = 0
	SCS |= (1<<5);	// OSCEN = 1
	
  // 2.  Wait for OSC ready
  //while(!SCS_bit.OSCSTAT);
	while (!(SCS|(1<<6)));
	
	
  // 3. Disconnect PLL
  //PLLCON_bit.PLLC = 0;
	PLLCON &= ~(1<<1);
  PLLFEED = 0xAA;
  PLLFEED = 0x55;
  // 4. Disable PLL
  //PLLCON_bit.PLLE = 0;
  PLLCON &= ~(1<<0);
  PLLFEED = 0xAA;
  PLLFEED = 0x55;
  // 5. Select souce clock for PLL
  //CLKSRCSEL_bit.CLKSRC = 1; // Selects the main oscillator as the PLL clock source.
  // 6. Set PLL settings 144 MHz
  //PLLCFG_bit.MSEL = 24-1;
  //PLLCFG_bit.NSEL = 2-1;
  PLLCFG = 0x00010023;
  PLLFEED = 0xAA;
  PLLFEED = 0x55;
  // 7. Enable PLL
  //PLLCON_bit.PLLE = 1;
  PLLCON |= 1<<0;
  PLLFEED = 0xAA;
  PLLFEED = 0x55;
  // 8. Wait for the PLL to achieve lock
  //while(!PLLSTAT_bit.PLOCK);
  //while (!(PLLSTAT &(1<<26)));
  
  // 9. Set clk divider settings
  CCLKCFG   = 6-1;        // 1/2 Fpll - 72 MHz
  USBCLKCFG = 6-1;            // 1/3 Fpll - 48 MHz
  PCLKSEL0 = PCLKSEL1 = 0;    // other pripherials - 18MHz
  // 10. Connect the PLL
  //PLLCON_bit.PLLC = 1;
  PLLCON |= 1<<1;
  PLLFEED = 0xAA;
  PLLFEED = 0x55;
}

int	main (void) {
	
	int		j;										// loop counter (stack variable)

	InitMAM();
	
	//InitClock();
	
	SCS |= 0x01;
	
	// Initialize the system
	Initialize();		
	
	// LPC-P2378
	/*FIO0DIR |= 1<<12; // p0.12 as output -> Stat led
	FIO0SET |= 1<<12; // off*/
	//LPC2378-STK
	FIO0DIR |= 1<<21; // P0.21 - power up the SD/MMC
	FIO0CLR |= 1<<21; // off
 
	
	// endless loop to toggle the two leds
	while (1) {
		
		for (j = 0; j < 50000; j++ );		// wait 500 msec
		//FIO0CLR |= 1<<12; // on
		FIO0SET |= 1<<21;
		for (j = 0; j < 50000; j++ );		// wait 500 msec
		//FIO0SET |= 1<<12; // off
		FIO0CLR |= 1<<21;
			
	}
}



/**********************************************************
                      Initialize
**********************************************************/

#define PLOCK 0x400

void Initialize(void)  {

   
	// Setting Multiplier and Divider values
  	/*PLLCFG=0x23;
  	feed();
  
	// Enabling the PLL 	
	PLLCON=0x1;
	feed();
  
	// Wait for the PLL to lock to set frequency
	while(!(PLLSTAT & PLOCK)) ;
  
	// Connect the PLL as the clock source
	PLLCON=0x3;
	feed();
  
	// Enabling MAM and setting number of clocks used for Flash memory fetch (4 cclks in this case)
	MAMCR=0x2;
	MAMTIM=0x4;*/
  
	// Setting peripheral Clock (pclk) to System Clock (cclk)
	//VPBDIV=0x1;

}


void feed(void)
{
  PLLFEED=0xAA;
  PLLFEED=0x55;
}






/*  Stubs for various interrupts (may be replaced later)  */
/*  ----------------------------------------------------  */

void IRQ_Routine (void) {
	while (1) ;	
}

void FIQ_Routine (void)  {
	while (1) ;	
}


void SWI_Routine (void)  {
	while (1) ;	
}


void UNDEF_Routine (void) {
	while (1) ;	
}










